/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmap;

import java.io.File;
//import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Properties;


/**
 * @author iwata
 *
 */
public class ACOFactory {
	  private static String libname = "org_agmodel_bioinfo_antmap_AntColonyJNI";
	  private boolean useJni = false;
	  
	  public ACOFactory() {
		  try {
				InputStream inputStream = this.getClass().getResourceAsStream("aco.properties");
				//InputStream inputStream = new FileInputStream("aco.properties");
				Properties properties = new Properties();
				properties.load(inputStream);
				useJni = Boolean.valueOf(properties.getProperty("use_jni")).booleanValue();
		  } catch (Exception ex) {
		  }  
	  }
	  
  public AntColonyOptimizer getAntColonyOptimizer(AcoParamsBean acoParamsBean,
                                                  double[][] dist) {
	  if (useJni && System.getProperty("os.name").startsWith("Windows") && new File(libname + ".dll").exists()) {
		  //System.out.println("Using " + libname + ".dll");
		  return new AntColonyJNI(acoParamsBean, dist);
	  }
	  return new AntColony(acoParamsBean, dist);
  }

  public AntColonyOptimizer getAntColonyOptimizer(AcoParamsBean acoParamsBean,
                                                  double[][] dist, long seed) {
	if (useJni && System.getProperty("os.name").startsWith("Windows") && new File(libname + ".dll").exists()) {
		  //System.out.println("Using " + libname + ".dll");
		  return new AntColonyJNI(acoParamsBean, dist, seed);
    }
    return new AntColony(acoParamsBean, dist, seed);
  }
}
