/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmap;

//import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Properties;

/**
 * @author iwata
 *
 * ̐ꂽRg̑}ev[gύX邽
 * EBhE > ݒ > Java > R[h > R[hƃRg
 */
public class AcoParamsBean implements java.io.Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1666077872937285707L;
	private double alpha = 1.0;
	private double beta  = 5.0;
	private double rho = 0.5;
	private double q = 10;
	private int n_iteration = 1000;
	private int n_ant = 20;
	private double init_tau = 0.00000001;
	private double p_fancy = 0.005;
	private double weight = 2.0;

	/**
	 * RXgN^
	 *
	 */
	public AcoParamsBean() {
		  try {
				InputStream inputStream = this.getClass().getResourceAsStream("aco.properties");
				//InputStream inputStream = new FileInputStream("aco.properties");
				Properties properties = new Properties();
				properties.load(inputStream);
				alpha = Double.valueOf(properties.getProperty("alpha")).doubleValue();
				beta = Double.valueOf(properties.getProperty("beta")).doubleValue();
				rho = Double.valueOf(properties.getProperty("rho")).doubleValue();
				q = Double.valueOf(properties.getProperty("q")).doubleValue();
				n_iteration = Integer.valueOf(properties.getProperty("n_iteration")).intValue();
				n_ant = Integer.valueOf(properties.getProperty("n_ant")).intValue();
				init_tau = Double.valueOf(properties.getProperty("init_tau")).doubleValue();
				p_fancy = Double.valueOf(properties.getProperty("p_fancy")).doubleValue();
				weight = Double.valueOf(properties.getProperty("weight")).doubleValue();
		  } catch (Exception ex) {
		  }
	}
	/**
	 * @return alpha
	 */
	public double getAlpha() {
		return alpha;
	}

	/**
	 * @return beta
	 */
	public double getBeta() {
		return beta;
	}

	/**
	 * @return init_tau
	 */
	public double getInit_tau() {
		return init_tau;
	}

	/**
	 * @return n_ant
	 */
	public int getN_ant() {
		return n_ant;
	}

	/**
	 * @return n_replicate
	 */
	public int getN_iteration() {
		return n_iteration;
	}

	/**
	 * @return p_fancy
	 */
	public double getP_fancy() {
		return p_fancy;
	}

	/**
	 * @return q
	 */
	public double getQ() {
		return q;
	}

	/**
	 * @return randomize (true or false)
	 */
	public boolean isRandomize() {
		if(p_fancy == 0) {
			return false;
		}
		return true;
	}

	/**
	 * @return rho
	 */
	public double getRho() {
		return rho;
	}

	/**
	 * set alpha
	 * @param d
	 */
	public void setAlpha(double d) {
		alpha = d;
	}

	/**
	 * set beta
	 * @param d
	 */
	public void setBeta(double d) {
		beta = d;
	}

	/**
	 * set init_tau
	 * @param d
	 */
	public void setInit_tau(double d) {
		init_tau = d;
	}

	/**
	 * set n_ant (number of ants)
	 * @param i
	 */
	public void setN_ant(int i) {
		n_ant = i;
	}

	/**
	 * set n_replicate (number of replicates)
	 * @param i
	 */
	public void setN_iteration(int i) {
		n_iteration = i;
	}

	/**
	 * set p_fancy (probability of fancy choice)
	 * @param d
	 */
	public void setP_fancy(double d) {
		p_fancy = d;
	}

	/**
	 * set q
	 * @param d
	 */
	public void setQ(double d) {
		q = d;
	}

	/**
	 * set rho
	 * @param d
	 */
	public void setRho(double d) {
		rho = d;
	}

	/**
	 * @return
	 */
	public double getWeight() {
		return weight;
	}

	/**
	 * @param d
	 */
	public void setWeight(double d) {
		weight = d;
	}

	public void show() {
		System.out.println("alpha=" + alpha);
		System.out.println("beta=" + beta);
		System.out.println("rho=" + rho);
		System.out.println("q=" + q);
		System.out.println("n_iteration=" + n_iteration);
		System.out.println("n_ant=" + n_ant);
		System.out.println("init_tau=" + init_tau);
		System.out.println("p_fancy=" + p_fancy);
		System.out.println("weight=" + weight);
	}

}
