/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmap;

import java.io.FileReader;
import java.io.BufferedReader;
import java.util.StringTokenizer;

/**
 * @author iwata
 *
 */
public class AcoParamsReader {
	public AcoParamsBean loadAcoParamsBean(String fileName) throws Exception {
		FileReader fr = new FileReader(fileName);
		AcoParamsBean acoParamsBean = new AcoParamsBean();

		try {
			BufferedReader br = new BufferedReader(fr);

			while(br.ready()) {
				String line = br.readLine();
				if(line.length() == 0) {
					continue;
				}
				if(line.startsWith("#")) {
					continue;
				}

				StringTokenizer st = new StringTokenizer(line, "=");
				String key = st.nextToken();
				if(key.equalsIgnoreCase("alpha")) {
					acoParamsBean.setAlpha(new Double(st.nextToken()).doubleValue());
				} else if(key.equalsIgnoreCase("beta")) {
					acoParamsBean.setBeta(new Double(st.nextToken()).doubleValue());
				} else if(key.equalsIgnoreCase("rho")) {
					acoParamsBean.setRho(new Double(st.nextToken()).doubleValue());
				} else if(key.equalsIgnoreCase("q")) {
					acoParamsBean.setQ(new Double(st.nextToken()).doubleValue());
				} else if(key.equalsIgnoreCase("n_iteration")) {
					acoParamsBean.setN_iteration(new Integer(st.nextToken()).intValue());
				} else if(key.equalsIgnoreCase("n_ant")) {
					acoParamsBean.setN_ant(new Integer(st.nextToken()).intValue());
				} else if(key.equalsIgnoreCase("init_tau")) {
					String temp = st.nextToken();
					//acoParamsBean.setInit_tau(new Double(st.nextToken()).doubleValue());
					acoParamsBean.setInit_tau(new Double(temp).doubleValue());
					//System.out.println(temp);
				} else if(key.equalsIgnoreCase("p_fancy")) {
					acoParamsBean.setP_fancy(new Double(st.nextToken()).doubleValue());
				} else if(key.equalsIgnoreCase("weight")) {
					acoParamsBean.setWeight(new Double(st.nextToken()).doubleValue());
				}
			}
			fr.close();
		}
		catch (Exception e) {
			throw e;
		}

		return acoParamsBean;
	}
}
