/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmap;

import java.io.FileWriter;
import java.util.Date;
import java.util.Locale;
import java.text.DateFormat;


/**
 * @author iwata
 *
 */
public class AcoParamsWriter {
	public void saveAcoParamsBean(String fileName, AcoParamsBean acoParamsBean) throws Exception {
		FileWriter fw = new FileWriter(fileName);
		String buf = "# Ant Colony Optimization Parameters\n";
		DateFormat dateFormat = DateFormat.getDateTimeInstance(DateFormat.FULL, DateFormat.LONG, Locale.US);
		buf += "# " + dateFormat.format(new Date()) + "\n\n";
		buf += "# alpha\n";
		buf += "alpha=" + acoParamsBean.getAlpha() + "\n\n";
		buf += "# beta\n";
		buf += "beta=" + acoParamsBean.getBeta() + "\n\n";
		buf += "# rho - trail pheromone decay coefficient\n";
		buf += "rho=" + acoParamsBean.getRho() + "\n\n";
		buf += "# Q - weight for pheromone added by ant\n";
		buf += "Q=" + acoParamsBean.getQ() + "\n\n";
		buf += "# n_iteration - the number of cycles of ACO process\n";
		buf += "n_iteration=" + acoParamsBean.getN_iteration() + "\n\n";
		buf += "# n_ant - the number of ants\n";
		buf += "n_ant=" + acoParamsBean.getN_ant() + "\n\n";
		buf += "# init_tau - initial value of tau, tau(0)\n";
		buf += "init_tau=" + acoParamsBean.getInit_tau() + "\n\n";
		buf += "# p_fancy - probability of fanciful choice of next stop\n";
		buf += "p_fancy=" + acoParamsBean.getP_fancy() + "\n\n";
		buf += "# weight - weight for the best tour\n";
		buf += "weight=" + acoParamsBean.getWeight() + "\n";
		fw.write(buf);
		fw.close();
	}

}
