/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmap;

/**
 * @author iwata
 *
 */
public class BootStrapTester {
	AcoMapFactory acoMapFactory;

	public BootStrapTester(AcoMapFactory acoMapFactory) {
		this.acoMapFactory = acoMapFactory;
	}

	public int [][] testOrder(SegDataBean segDataBean, MapData mapData, int maxIteration, int n_confirm, int bootStrapIteration) {
		int [] markerIndex = mapData.getMarkerIndexArray();
		int n_marker = markerIndex.length;
		int [][] count = new int [n_marker][n_marker];
		for(int i = 0; i < bootStrapIteration; i++ ) {
			TourBean bestTour = acoMapFactory.getBootStrapMap(segDataBean, markerIndex, maxIteration, n_confirm);
			//debug
			//System.out.print((i + 1) + ": ");
			//bestTour.showOrder();
			//debug
			int [] bestOrder = bestTour.getTourOrder();
			for(int j = 0; j < bestOrder.length; j++ )
				if(mapData.getMarkerLocation(j) != mapData.getMarkerLocation(bestOrder[j]))
					count[j][bestOrder[j]] ++;
				else
					count[j][j] ++;
		}
		for(int i = 0; i < n_marker; i++ ) {
			mapData.setMarkerBootProb(i, ((double) count[i][i]) / bootStrapIteration);
		}
		return count;
	}
}
