/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmap;

import org.agmodel.bioinfo.segregation.MapFunction;

/**
 * @author iwata
 *
 */
public class LGFactory {
	public static final int ME_NNL = 0;
	public static final int ME_ALL = 1;
	public static final int CR_R = 0;
	public static final int CR_DIST = 1;
	public static final int CR_P = 2;
	public static final int CR_LOD = 3;
	public static final String ME_NNL_NAME = "nearest neighboring locus";
	public static final String ME_ALL_NAME = "all combinations";
	public static final String CR_R_NAME = "recombination";
	public static final String CR_DIST_NAME = "distance (cM)";
	public static final String CR_P_NAME = "p value";
	public static final String CR_LOD_NAME = "LOD";

	private MapFunction mapFunction = new MapFunction();

	public String getMethodName(int method) {
		if(method == ME_ALL)
			return ME_ALL_NAME;
		else
			return ME_NNL_NAME;
	}

	public String getCriterionName(int criterion) {
		if(criterion == CR_DIST)
			return CR_DIST_NAME;
		else if(criterion == CR_P)
			return CR_P_NAME;
		else if(criterion == CR_LOD)
			return CR_LOD_NAME;
		else
			return CR_R_NAME;
	}

	public GroupExtractor getGroupExtractor(SegDataBean segDataBean, int method, int criterion, double thresh, int n_min) {
		LinkageAnalyzer linkageAnalyzer = new LinkageAnalyzer(segDataBean);
		double [][] mat;
		boolean threshIsUpper;
		double new_thresh;
		if(criterion == CR_P) {
			mat = linkageAnalyzer.getP();
			threshIsUpper = true;
			new_thresh = thresh;
		} else if(criterion == CR_LOD) {
			mat = linkageAnalyzer.getLod();
			threshIsUpper = false;
			new_thresh = thresh;
		} else if(criterion == CR_DIST) {
			mat = linkageAnalyzer.getR();
			threshIsUpper = true;
			new_thresh = mapFunction.getRate(thresh / 100);
		} else { //CR_R (Default)
			mat = linkageAnalyzer.getR();
			threshIsUpper = true;
			new_thresh = thresh;
		}
		int [] groupingIndex;
		if(method == ME_ALL)
			groupingIndex = new GroupDivider().getGroupingIndex(mat, new_thresh, threshIsUpper);
		else // ME_NNL (Default)
			groupingIndex = new GroupDivider().getGroupingIndexNNL(mat, new_thresh, threshIsUpper);
		return new GroupExtractor(groupingIndex, n_min);
	}

	/**
	 * @return
	 */
	public int getMapFunction() {
		return mapFunction.getFunction();
	}

	/**
	 * @param i
	 */
	public void setMapFunction(int i) {
		mapFunction.setFunction(i);
	}

}
