/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmap;

import java.util.Arrays;

/**
 * @author iwata
 *
 */
public class OrderAligner {

	public void align(int [] tourOrder, double r[][]) {
		alignMarkersInSameLocation(tourOrder, r);
		int [] dummyOrder = new int [tourOrder.length];
		dummyOrder[0] = tourOrder[0];
		for(int i = 1; i < tourOrder.length; i++)
			if(r[tourOrder[i]][tourOrder[i - 1]] == 0)
				dummyOrder[i] = dummyOrder[i - 1];
			else
				dummyOrder[i] = tourOrder[i];
		if(count(dummyOrder, r) < 0)
			reverse(tourOrder);
		alignMarkersInSameLocation(tourOrder, r);
	}

	public void align_simple(int [] tourOrder) {
		if(count(tourOrder) < 0)
			reverse(tourOrder);
	}

	private void alignMarkersInSameLocation(int [] tourOrder, double r [][]) {
		int count = 0;
		for(int i = 1; i < tourOrder.length; i++ ) {
			if(r[tourOrder[i]][tourOrder[i - 1]] == 0)
				count ++;
			if(count > 0 && r[tourOrder[i]][tourOrder[i - 1]] > 0) {
				Arrays.sort(tourOrder, i - 1 - count, i - 1);
				count = 0;
			}
		}
		if(count > 0)
			Arrays.sort(tourOrder, tourOrder.length - 1 - count, tourOrder.length - 1);
	}

	/*
	private void reverse(int [] ia) {
		int temp;
		for(int i = 0; i < (double)ia.length / 2; i++ ) {
			temp = ia[i];
			ia[i] = ia[ia.length - 1 - i];
			ia[ia.length - 1 - i] = temp;
		}
	}
	*/

	private void reverse(int [] ia) {
		int [] rev = new int [ia.length];
		for(int i = 0; i < ia.length; i++ )
			rev[i] = ia[ia.length - 1 - i];
		for(int i = 0; i < ia.length; i++ )
			ia[i] = rev[i];
	}

	private int count(int [] tourOrder, double r [][]) {
		int count = 0;
		for(int i = 1; i < tourOrder.length; i++ )
			if(r[tourOrder[i]][tourOrder[i - 1]] != 0)
				if(tourOrder[i] > tourOrder[i - 1])
					count ++;
				else
					count --;
		return count;
	}

	private int count(int [] tourOrder) {
		int count = 0;
		for(int i = 1; i < tourOrder.length; i++ )
			if(tourOrder[i] > tourOrder[i - 1])
				count ++;
			else
				count --;
		return count;
	}
}
