/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmapgui;

import javax.swing.JTextArea;
import javax.swing.JProgressBar;
import java.text.DecimalFormat;
import org.agmodel.bioinfo.common.StringFormat;
import org.agmodel.bioinfo.antmap.SegDataBean;
import org.agmodel.bioinfo.antmap.MapData;
import org.agmodel.bioinfo.antmap.OrderingOptionBean;
import org.agmodel.bioinfo.antmap.AcoParamsBean;
import org.agmodel.bioinfo.antmap.MarkerBean;
import org.agmodel.bioinfo.antmap.RMatrixFactory;
import org.agmodel.bioinfo.antmap.OrderingCriterion;
import org.agmodel.bioinfo.antmap.NMatrixFactory;
import org.agmodel.bioinfo.antmap.LDMatrixFactory;
import org.agmodel.bioinfo.antmap.AntColonyOptimizer;
import org.agmodel.bioinfo.antmap.ACOFactory;
import org.agmodel.bioinfo.antmap.TourBean;
import org.agmodel.bioinfo.antmap.OrderAligner;

/**
 * <p>^Cg: </p>
 * <p>: </p>
 * <p>쌠: Copyright (c) 2004</p>
 * <p>Ж: </p>
 * @author 
 * @version 1.0
 */
/**
 * Thread for bootstrap test
 * <p>Title: AntMap</p>
 * <p>Explanation: Linkage mapping software based on ACO algorithm</p>
 * <p>Copyright: Copyright (c) 2004 NARO</p>
 * <p>Company name: NARO</p>
 * @author Hiroyoshi IWATA
 * @version 1.1
 */
public class BootstrapTest extends Thread {
  JTextArea log;
  JTextArea result;
  DrawMapPanel map;
  JProgressBar progress;
  MapData [] mapData;
  SegDataBean segDataBean;
  OrderingOptionBean orderingOptionBean;
  AcoParamsBean acoParamsBean;
  ThreadIndicator threadIndicator;

  public BootstrapTest(JTextArea log,
                       JTextArea result,
                       DrawMapPanel map,
                       JProgressBar progress,
                       MapData [] mapData,
                       SegDataBean segDataBean,
                       OrderingOptionBean orderingOptionBean,
                       AcoParamsBean acoParamsBean,
                       ThreadIndicator threadIndicator) {
    this.log = log;
    this.result = result;
    this.map = map;
    this.progress = progress;
    this.mapData = mapData;
    this.segDataBean = segDataBean;
    this.orderingOptionBean = orderingOptionBean;
    this.acoParamsBean = acoParamsBean;
    this.threadIndicator = threadIndicator;
  }

  public void run() {
    if(!threadIndicator.isInService()) {
      threadIndicator.setInService(true);
      long start = System.currentTimeMillis();
      bootstrap();
      map.drawMap(mapData);
      long stop = System.currentTimeMillis();
      log.append("    ...ok\n");
      log.append("    run time: " + ((double)(stop - start) / 1000) + " sec\n");
      log.setCaretPosition(log.getText().length());
      progress.setValue(0);
      threadIndicator.setInService(false);
    }
  }

  private void bootstrap() {
    int nGroup = mapData.length;
    int groupIndex = orderingOptionBean.getGroupIndex();
    boolean target_all = (groupIndex == -1);
    for(int i = 0; i < nGroup; i++ ) {
      if ((target_all || (groupIndex == i)) && mapData[i] != null) {
        log.append("    Group" + (i + 1) + ": ");
        log.setCaretPosition(log.getText().length());
        result.append("------------\n");
        result.append("Group" + (i + 1) + ": \n");
        result.setCaretPosition(result.getText().length());

        int [][] count = testOrder(mapData[i]);
        log.append(orderingOptionBean.getNBootstrap() + "trials\n");
        log.setCaretPosition(log.getText().length());
        DecimalFormat decimalFormat = new DecimalFormat("##0.0");
        StringFormat stringFormat = new StringFormat();
        for (int j = 0; j < mapData[i].getNMarker(); j++) {
          MarkerBean marker = mapData[i].getMarker(j);
          result.append("    ");
          result.append(stringFormat.format(new Integer(marker.getIndex()).
                                            toString(), 5, true));
          result.append(stringFormat.format(marker.getName(), 12, true));
          result.append(stringFormat.format(decimalFormat.format(marker.getBootProb() * 100), 8, false));
          result.append("\n");
        }
        result.setCaretPosition(result.getText().length());
      }
    }

  }

  private int [][] testOrder(MapData mapData) {
    int [] markerIndex = mapData.getMarkerIndexArray();
    int nMarker = markerIndex.length;
    int [][] count = new int [nMarker][nMarker];
    int nBootstrap = orderingOptionBean.getNBootstrap();
    progress.setMinimum(0);
    progress.setMaximum(nBootstrap);
    for(int i = 0; i < nBootstrap; i++ ) {
      int [] bestOrder = getBootstrapMap(markerIndex);
      for(int j = 0; j < bestOrder.length; j++ )
        if(mapData.getMarkerLocation(j) != mapData.getMarkerLocation(bestOrder[j]))
          count[j][bestOrder[j]] ++;
        else
          count[j][j] ++;
      progress.setValue(i + 1);
    }

    for(int i = 0; i < nMarker; i++ )
      mapData.setMarkerBootProb(i, (double) count[i][i] / nBootstrap);
    return count;
  }


  private int [] getBootstrapMap(int [] markerIndex) {
    double [][] r = new RMatrixFactory().getBootStrappedR(segDataBean, markerIndex);
    double [][] dist;
    if(orderingOptionBean.getCriterion() == OrderingCriterion.OC_LL) {
      int [][] n = new NMatrixFactory().getN(segDataBean, markerIndex);
      dist = new LDMatrixFactory().convertRN2LD(r, n);
    } else {
      dist = r;
    }
    AntColonyOptimizer aco = new ACOFactory().getAntColonyOptimizer(acoParamsBean, dist);
    TourBean bestTour = new TourBean();
    int count = 0;
    for(int i = 0; i < orderingOptionBean.getMaxIteration(); i++ ) {
      TourBean tempTour = aco.findBestTour();
      if(bestTour.getTourLength() == tempTour.getTourLength())
        count ++;
      else if(bestTour.getTourLength() > tempTour.getTourLength()) {
        bestTour = tempTour;
        count = 0;
      }
      if(count == orderingOptionBean.getNConfirm())
        break;
    }
    new OrderAligner().align(bestTour.getTourOrder(),r);
    return bestTour.getTourOrder();
  }

}
