/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmapgui;

import javax.swing.*;
import java.awt.Dimension;
import java.awt.event.*;


/**
 * Panel for inputing a parameter in double
 * <p>Title: AntMap</p>
 * <p>Explanation: Linkage mapping software based on ACO algorithm</p>
 * <p>Copyright: Copyright (c) 2004 NARO</p>
 * <p>Company name: NARO</p>
 * @author Hiroyoshi IWATA
 * @version 1.1
 */
public class DblParamPanel extends JPanel {
  public static final int VR_ALL = 0;
  public static final int VR_POSITIVE_AND_ZERO = 1;
  public static final int VR_POSITIVE = 2;
  public static final int VR_NEGATIVE_AND_ZERO = 3;
  public static final int VR_NEGATIVE = 4;

  private int range = VR_ALL;
  private String text = "NA";
  private double value = 0.0;

  JLabel jLabelText = new JLabel();
  JLabel jLabelValue = new JLabel();
  JTextField jTextField = new JTextField();
  JButton jButtonAlter = new JButton();

  public DblParamPanel() {
    try {
      jbInit();
      jLabelValue.setText(new Double(value).toString());
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public DblParamPanel(String name, double value, int range) {
    this.text = name;
    this.value = value;
    this.range = range;
    try {
      jbInit();
      jLabelValue.setText(new Double(value).toString());
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public String getText() {
    return text;
  }

  public void setText(String text) {
    this.text = text;
    jLabelText.setText(text);
  }

  public double getValue() {
    return value;
  }

  public void setValue(double value) {
    this.value = value;
    jLabelValue.setText(new Double(value).toString());
  }

  public int getRange() {
    return range;
  }

  public void setRange(int range) {
    this.range = range;
  }

  private void jbInit() throws Exception {
    jLabelText.setText(text);
    jLabelValue.setText("0.0");
    jTextField.setPreferredSize(new Dimension(60, 21));
    jTextField.setText("");
    jButtonAlter.setText("Alter");
    jButtonAlter.addActionListener(new DblParamPanel_jButtonAlter_actionAdapter(this));
    this.add(jLabelText, null);
    this.add(jLabelValue, null);
    this.add(jTextField, null);
    this.add(jButtonAlter, null);
  }

  void jButtonAlter_actionPerformed(ActionEvent e) {
    double tempValue;
    try {
      tempValue = Double.parseDouble(jTextField.getText());
      if(range == VR_POSITIVE_AND_ZERO && tempValue < 0) {
        JOptionPane.showMessageDialog(this, "Value must be positive or zero", "Invalid input", JOptionPane.ERROR_MESSAGE);
      } else if(range == VR_POSITIVE && tempValue <= 0) {
        JOptionPane.showMessageDialog(this, "Value must be positive", "Invalid input", JOptionPane.ERROR_MESSAGE);
      } else if(range == VR_NEGATIVE_AND_ZERO && tempValue > 0) {
        JOptionPane.showMessageDialog(this, "Value must be negative or zero", "Invalid input", JOptionPane.ERROR_MESSAGE);
      } else if(range == VR_NEGATIVE && tempValue >= 0) {
        JOptionPane.showMessageDialog(this, "Value must be negative", "Invalid input", JOptionPane.ERROR_MESSAGE);
      } else {
        value = tempValue;
        jLabelValue.setText(new Double(value).toString());
      }
    } catch (Exception ex) {
      JOptionPane.showMessageDialog(this, ex, "Invalid input", JOptionPane.ERROR_MESSAGE);
    }
    jTextField.setText("");

  }

}

class DblParamPanel_jButtonAlter_actionAdapter implements java.awt.event.ActionListener {
  DblParamPanel adaptee;

  DblParamPanel_jButtonAlter_actionAdapter(DblParamPanel adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jButtonAlter_actionPerformed(e);
  }
}
