/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmapgui;

import javax.swing.JPanel;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Font;
import java.text.DecimalFormat;
import java.io.FileOutputStream;
import java.io.File;
import com.sun.image.codec.jpeg.JPEGImageEncoder;
import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGEncodeParam;
import org.agmodel.bioinfo.antmap.*;

/**
 * <p>^Cg: </p>
 * <p>: </p>
 * <p>쌠: Copyright (c) 2004</p>
 * <p>Ж: </p>
 * @author 
 * @version 1.0
 */

public class DrawMapPanel extends JPanel {
  public static final int MAX_WIDTH = 10000;
  public static final int MAX_HEIGHT = 10000;
  private DrawMapOptionBean drawMapOptionBean;
  private int chromWidth = 10;
  private int tickLength = chromWidth / 2;
  private int kinkLength = 15;
  private int spaceLength = 5;
  private int spaceHeight = 10;
  private String format = "###########0.0";
  private int pWidth;
  private int pHeight;
  private BufferedImage bufferedImage;

  private Graphics2D gBufferedImage;

  public DrawMapPanel() {
    pWidth = 640;
    pHeight = 480;
    drawMapOptionBean = new DrawMapOptionBean();
    bufferedImage = new BufferedImage(pWidth, pHeight, BufferedImage.TYPE_3BYTE_BGR);
    gBufferedImage = bufferedImage.createGraphics();
    clear();
  }

  public void setDrawMapOptionBean(DrawMapOptionBean drawMapOptionBean) {
    this.drawMapOptionBean = drawMapOptionBean;
  }

  public DrawMapOptionBean getDrawMapOptionBean() {
    return drawMapOptionBean;
  }

  public void clear() {
    gBufferedImage.setColor(Color.WHITE);
    gBufferedImage.fillRect(0, 0, pWidth, pHeight);
  }

  public void saveImage(File file, float quality) throws Exception {
    FileOutputStream fileOutputStream = new FileOutputStream(file);
    JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(fileOutputStream);
    JPEGEncodeParam eparam = encoder.getDefaultJPEGEncodeParam(bufferedImage);
    eparam.setQuality(quality, false);
    encoder.setJPEGEncodeParam(eparam);
    encoder.encode(bufferedImage);
    fileOutputStream.close();
  }

  public void drawMap(MapData [] mapData) {
    int mapSpacing = drawMapOptionBean.getMapSpacing();
    double scaleFactor = drawMapOptionBean.getScaleFactor();
    int sx = drawMapOptionBean.getMarginSize();
    int sy = drawMapOptionBean.getMarginSize();

    //Œ̃f[^
    if(mapData != null) {
      int maxHeight = 0;
      int cumWidth = 0;
      int count = 0;
      for (int i = 0; i < mapData.length; i++) {
        if(mapData[i] != null) {
          int height = getImageHeight(mapData[i]);
          if (height > maxHeight)
            maxHeight = height;
          cumWidth += getImageWidth(mapData[i]);
          if(count > 0) cumWidth += drawMapOptionBean.getMapSpacing();
          count ++;
        }
      }

      //plTCY̌
      int groupNameSpaceHeight = getGroupNameSpaceHeight();
      setPanelSize(sx + cumWidth + sx, sy + groupNameSpaceHeight + maxHeight + sy);

      //}bv̕`
      /*
      count = 0;
      for (int i = 0; i < mapData.length; i++) {
        if (mapData[i] != null) {
          drawEachMap(mapData[i], sx + mapSpacing * count, sy);
          count ++;
        }
      }
      */
     for (int i = 0; i < mapData.length; i++) {
       if (mapData[i] != null) {
         sx += getImageWidthLeftSide(mapData[i]);
         drawEachMap(mapData[i], sx, sy);
         sx += getImageWidthRightSide(mapData[i]) + drawMapOptionBean.getMapSpacing();
       }
     }
    }
  }

  private int getGroupNameSpaceHeight() {
    gBufferedImage.setFont(drawMapOptionBean.getFontGroupName());
    FontMetrics fm = gBufferedImage.getFontMetrics();
    int height = fm.getHeight();
    height += spaceHeight;
    gBufferedImage.setFont(drawMapOptionBean.getFontMarkerInfo());
    fm = gBufferedImage.getFontMetrics();
    height += fm.getAscent() / 2;
    return height;
  }

  private int getImageHeight(MapData mapData) {
    int py = - Integer.MAX_VALUE;
    gBufferedImage.setFont(drawMapOptionBean.getFontMarkerInfo());
    FontMetrics fm = gBufferedImage.getFontMetrics();
    for(int i = 0; i < mapData.getNMarker(); i++ ) {
      int ty = (int)(mapData.getMarkerLocation(i) * drawMapOptionBean.getScaleFactor());
      if(ty < py + fm.getHeight()) {
        ty = py + fm.getHeight();
      }
      py = ty;
    }
    py += fm.getAscent() / 2 + fm.getDescent();
    return py;
  }

  private int getImageWidth(MapData mapData) {
    return getImageWidthLeftSide(mapData) + getImageWidthRightSide(mapData);
  }

  private int getImageWidthLeftSide(MapData mapData) {
    int max = 0;
    gBufferedImage.setFont(drawMapOptionBean.getFontMarkerInfo());
    FontMetrics fm = gBufferedImage.getFontMetrics();
    DecimalFormat df = new DecimalFormat(format);
    for(int i = 0; i < mapData.getNMarker(); i++ ) {
      int length = fm.stringWidth(df.format(mapData.getMarkerLocation(i)));
      if(length > max)
        max = length;
    }
    return max + tickLength + kinkLength + spaceLength;
  }

  private int getImageWidthRightSide(MapData mapData) {
    int max = 0;
    gBufferedImage.setFont(drawMapOptionBean.getFontMarkerInfo());
    FontMetrics fm = gBufferedImage.getFontMetrics();
    DecimalFormat df = new DecimalFormat(format);
    for(int i = 0; i < mapData.getNMarker(); i++ ) {
      int length = fm.stringWidth(mapData.getMarkerName(i));
      if(drawMapOptionBean.isShowBootstrapValue())
        length += fm.stringWidth(" (" + df.format(mapData.getMarkerBootProb(i)) + ")");
      if(length > max)
        max = length;
    }
    return max + tickLength + kinkLength + spaceLength;
  }

  public void drawEachMap(MapData mapData, int sx, int sy) {
    double scaleFactor = drawMapOptionBean.getScaleFactor();

    //F̖̕`
    gBufferedImage.setColor(Color.BLACK);
    gBufferedImage.setFont(drawMapOptionBean.getFontGroupName());
    FontMetrics fm = gBufferedImage.getFontMetrics();
    gBufferedImage.drawString(mapData.getName(),
                              sx - fm.stringWidth(mapData.getName()) / 2,
                              sy + fm.getAscent());

    sy += getGroupNameSpaceHeight();

    //F̖{̂̃h[
    gBufferedImage.setColor(Color.GRAY);
    gBufferedImage.fillRect(sx - chromWidth / 2, sy, chromWidth, (int)(mapData.getLength() * scaleFactor));
    gBufferedImage.setColor(Color.BLACK);
    gBufferedImage.drawRect(sx - chromWidth / 2, sy, chromWidth, (int)(mapData.getLength() * scaleFactor));
    //}[J[̈ʒuƖO

    DecimalFormat df = new DecimalFormat(format);
    int py = - Integer.MAX_VALUE;
    double ploc = - Double.MAX_VALUE;
    int x_offset = tickLength + kinkLength + spaceLength;

    gBufferedImage.setFont(drawMapOptionBean.getFontMarkerInfo());
    fm = gBufferedImage.getFontMetrics();

    for(int i = 0; i < mapData.getNMarker(); i++ ) {
      //}[J[ʒuɍ݂
      int ty = sy + (int)(mapData.getMarkerLocation(i) * scaleFactor);
      gBufferedImage.drawLine(sx - tickLength, ty, sx + tickLength, ty);
      int iy = ty;
      //}[J[ވʒu߂
      if(iy < py + fm.getHeight()) {
        iy = py + fm.getHeight();
      }

      gBufferedImage.drawLine(sx + tickLength, ty,
                              sx + tickLength + kinkLength, iy);
      gBufferedImage.drawString(mapData.getMarkerName(i), sx + x_offset, iy + fm.getAscent() / 2);
      if(drawMapOptionBean.isShowBootstrapValue()) {
        int alpha = fm.stringWidth(mapData.getMarkerName(i) + " ");
        gBufferedImage.drawString(" (" + df.format(mapData.getMarkerBootProb(i) * 100) + ")",
                                  sx + x_offset + alpha,
                                  iy + fm.getAscent() / 2);
      }
      double loc = mapData.getMarkerLocation(i);
      if(loc != ploc) {
        /*
        gBufferedImage.drawLine(sx + tickLength, ty,
                                sx + tickLength + kinkLength, iy);
        */
        gBufferedImage.drawLine(sx - tickLength, ty,
                                sx - tickLength - kinkLength, iy);
        String temp = df.format(loc);
        gBufferedImage.drawString(temp,
                                  sx - x_offset - fm.stringWidth(temp), iy + fm.getAscent() / 2);
      }
      /* else
        gBufferedImage.drawLine(sx + tickLength + kinkLength, iy - fm.getHeight() - fm.getAscent() / 2,
                                sx + tickLength + kinkLength, iy + fm.getAscent() / 2);
        */
      ploc = loc;
      py = iy;
    }
  }

  public void setPanelSize(int width, int height) {
    pWidth = width;
    pHeight = height;
    if(pWidth > MAX_WIDTH)
      pWidth = MAX_WIDTH;
    if(pHeight > MAX_HEIGHT)
      pHeight = MAX_HEIGHT;
    bufferedImage = null;
    gBufferedImage = null;
    bufferedImage = new BufferedImage(pWidth, pHeight, BufferedImage.TYPE_3BYTE_BGR);
    gBufferedImage = bufferedImage.createGraphics();
    clear();
  }

  public Dimension getPreferredSize() {
    return new Dimension(pWidth, pHeight);
  }

  //`
  public void paintComponent(Graphics g) {
    g.drawImage(bufferedImage, 0, 0, this);
  }

  public int getPanelWidth() {
    return pWidth;
  }

  public int getPanelHeight() {
    return pHeight;
  }

}
