/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmapgui;

import javax.swing.*;
import java.awt.*;
import javax.swing.event.*;
import org.agmodel.bioinfo.segregation.MapFunction;
import java.awt.event.*;
import org.agmodel.bioinfo.antmap.*;

/**
 * <p>^Cg: </p>
 * <p>: </p>
 * <p>쌠: Copyright (c) 2004</p>
 * <p>Ж: </p>
 * @author 
 * @version 1.0
 */

public class GroupingOptionPanel extends JPanel {
  private GroupingOptionBean groupingOptionBean = new GroupingOptionBean();

  JPanel jPanel2 = new JPanel();
  JLabel jLabel2 = new JLabel();
  JRadioButton jRadioButtonR = new JRadioButton();
  JRadioButton jRadioButtonDist = new JRadioButton();
  JRadioButton jRadioButtonP = new JRadioButton();
  JRadioButton jRadioButtonLOD = new JRadioButton();
  ButtonGroup buttonGroup1 = new ButtonGroup();
  GridLayout gridLayout1 = new GridLayout();
  DblParamPanel dppThreshold = new DblParamPanel();
  IntParamPanel ippMinNMarker = new IntParamPanel();
  RadioButtonPanel rbpMethod = new RadioButtonPanel();
  RadioButtonPanel rbpMapFunction = new RadioButtonPanel();


  public GroupingOptionPanel() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public GroupingOptionBean getGroupingOptionBean() {
    groupingOptionBean.setMethod(rbpMethod.getSelectedIndex());
    groupingOptionBean.setCriterion(getCriterion());
    groupingOptionBean.setMapFunction(rbpMapFunction.getSelectedIndex());
    groupingOptionBean.setThreshold(dppThreshold.getValue());
    groupingOptionBean.setMinNMarker(ippMinNMarker.getValue());
    return groupingOptionBean;
  }

  public void setGroupingOptionBean(GroupingOptionBean groupingOptionBean) {
    this.groupingOptionBean = groupingOptionBean;
    rbpMethod.setSelectedIndex(this.groupingOptionBean.getMethod());
    setCriterion(this.groupingOptionBean.getCriterion());
    rbpMapFunction.setSelectedIndex(this.groupingOptionBean.getMapFunction());
    dppThreshold.setValue(this.groupingOptionBean.getThreshold());
    ippMinNMarker.setValue(this.groupingOptionBean.getMinNMarker());
  }

  public int getCriterion() {
    if(jRadioButtonR.isSelected())
      return LGFactory.CR_R;
    else if(jRadioButtonDist.isSelected())
      return LGFactory.CR_DIST;
    else if(jRadioButtonP.isSelected())
      return LGFactory.CR_P;
    else if(jRadioButtonLOD.isSelected())
      return LGFactory.CR_LOD;
    return -1;
  }

  public void setCriterion(int criterion) {
    if(criterion == LGFactory.CR_R)
      jRadioButtonR.setSelected(true);
    else if(criterion == LGFactory.CR_DIST)
      jRadioButtonDist.setSelected(true);
    else if(criterion == LGFactory.CR_P)
      jRadioButtonP.setSelected(true);
    else if(criterion == LGFactory.CR_LOD)
      jRadioButtonLOD.setSelected(true);
  }

  private void jbInit() throws Exception {
    rbpMethod.setText("Method: ");
    rbpMethod.addItem(LGFactory.ME_NNL_NAME);
    rbpMethod.addItem(LGFactory.ME_ALL_NAME);
    rbpMethod.setSelectedIndex(groupingOptionBean.getMethod());
    jLabel2.setText("Criterion: ");
    jRadioButtonR.setSelected(true);
    jRadioButtonR.setText(LGFactory.CR_R_NAME);
    jRadioButtonDist.setText(LGFactory.CR_DIST_NAME);
    jRadioButtonDist.addChangeListener(new GroupingOptionPanel_jRadioButtonDist_changeAdapter(this));
    jRadioButtonP.setText(LGFactory.CR_P_NAME);
    jRadioButtonLOD.setText(LGFactory.CR_LOD_NAME);
    rbpMapFunction.setText("Map function: ");
    rbpMapFunction.addItem(MapFunction.MF_HALDANE_NAME);
    rbpMapFunction.addItem(MapFunction.MF_KOSAMBI_NAME);
    rbpMapFunction.setSelectedIndex(groupingOptionBean.getMapFunction());
    rbpMapFunction.setEnabled(false);
    dppThreshold.setText("Threshold: ");
    dppThreshold.setValue(groupingOptionBean.getThreshold());
    dppThreshold.setRange(DblParamPanel.VR_POSITIVE);
    ippMinNMarker.setText("Min # markers: ");
    ippMinNMarker.setValue(groupingOptionBean.getMinNMarker());
    ippMinNMarker.setRange(IntParamPanel.VR_POSITIVE);
    this.setLayout(gridLayout1);
    gridLayout1.setColumns(1);
    gridLayout1.setRows(6);
    this.add(rbpMethod, null);
    this.add(jPanel2, null);
    jPanel2.add(jLabel2, null);
    jPanel2.add(jRadioButtonR, null);
    jPanel2.add(jRadioButtonDist, null);
    jPanel2.add(jRadioButtonP, null);
    jPanel2.add(jRadioButtonLOD, null);
    buttonGroup1.add(jRadioButtonR);
    buttonGroup1.add(jRadioButtonDist);
    buttonGroup1.add(jRadioButtonP);
    buttonGroup1.add(jRadioButtonLOD);
    this.add(rbpMapFunction, null);
    this.add(dppThreshold, null);
    this.add(ippMinNMarker, null);
  }

  void jRadioButtonDist_stateChanged(ChangeEvent e) {
    if(jRadioButtonDist.isSelected()) {
      rbpMapFunction.setEnabled(true);
    } else {
      rbpMapFunction.setEnabled(false);
    }
  }

}

class GroupingOptionPanel_jRadioButtonDist_changeAdapter implements javax.swing.event.ChangeListener {
  GroupingOptionPanel adaptee;

  GroupingOptionPanel_jRadioButtonDist_changeAdapter(GroupingOptionPanel adaptee) {
    this.adaptee = adaptee;
  }
  public void stateChanged(ChangeEvent e) {
    adaptee.jRadioButtonDist_stateChanged(e);
  }
}
