/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.antmapgui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import org.agmodel.bioinfo.segregation.MapFunction;
import org.agmodel.bioinfo.antmap.*;
import javax.swing.event.*;

/**
 * <p>^Cg: </p>
 * <p>: </p>
 * <p>쌠: Copyright (c) 2004</p>
 * <p>Ж: </p>
 * @author 
 * @version 1.0
 */

public class OrderingOptionPanel extends JPanel {
  private OrderingOptionBean orderingOptionBean = new OrderingOptionBean();

  GridLayout gridLayout1 = new GridLayout();
  ButtonGroup buttonGroup1 = new ButtonGroup();
  //IntParamPanel ippMaxIteration = new IntParamPanel();
  //IntParamPanel ippNConfirm = new IntParamPanel();
  IntParamPanel ippNIteration = new IntParamPanel();
  IntParamPanel ippNBootstrap = new IntParamPanel();
  RadioButtonPanel rbpMapFunction = new RadioButtonPanel();
  RadioButtonPanel rbpCriterion = new RadioButtonPanel();
  ComboBoxPanel cbpTargetedGroup = new ComboBoxPanel();

  public OrderingOptionPanel() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public OrderingOptionBean getOrderingOptionBean() {
    //orderingOptionBean.setMaxIteration(ippMaxIteration.getValue());
    //orderingOptionBean.setNConfirm(ippNConfirm.getValue());
    orderingOptionBean.setMaxIteration(ippNIteration.getValue());
    orderingOptionBean.setNBootstrap(ippNBootstrap.getValue());
    orderingOptionBean.setMapFunction(rbpMapFunction.getSelectedIndex());
    orderingOptionBean.setCriterion(rbpCriterion.getSelectedIndex());
    orderingOptionBean.setGroupIndex(getGroupIndex());
    return orderingOptionBean;
  }

  public void setOrderingOptionBean(OrderingOptionBean orderingOptionBean) {
    this.orderingOptionBean = orderingOptionBean;
    //ippMaxIteration.setValue(this.orderingOptionBean.getMaxIteration());
    //ippNConfirm.setValue(this.orderingOptionBean.getNConfirm());
    ippNIteration.setValue(this.orderingOptionBean.getMaxIteration());
    ippNBootstrap.setValue(this.orderingOptionBean.getNBootstrap());
    rbpMapFunction.setSelectedIndex(this.orderingOptionBean.getMapFunction());
    rbpCriterion.setSelectedIndex(this.orderingOptionBean.getCriterion());
    //cbpTargetedGroup͕ύXȂ
  }

  public int getGroupIndex() {
    return cbpTargetedGroup.getSelectedIndex() - 1;
  }

  public void setGroupNames(String [] groupName) {
    cbpTargetedGroup.removeAllItems();
    cbpTargetedGroup.addItem("All");
    if(groupName != null)
      cbpTargetedGroup.addItems(groupName);
    cbpTargetedGroup.setSelectedIndex(0);
  }

  private void jbInit() throws Exception {
    rbpCriterion.setText("Criterion: ");
    rbpCriterion.addItem(OrderingCriterion.OC_LL_NAME);
    rbpCriterion.addItem(OrderingCriterion.OC_SARF_NAME);
    rbpCriterion.setSelectedIndex(orderingOptionBean.getCriterion());
    //ippMaxIteration.setText("Max # iterations: ");
    //ippMaxIteration.setValue(orderingOptionBean.getMaxIteration());
    //ippMaxIteration.setRange(IntParamPanel.VR_POSITIVE);
    ippNIteration.setText("# runs of locus ordering (to confirm the solution): ");
    ippNIteration.setValue(orderingOptionBean.getMaxIteration());
    ippNIteration.setRange(IntParamPanel.VR_POSITIVE);
    //ippNConfirm.setText("# iterations to confirm order: ");
    //ippNConfirm.setValue(orderingOptionBean.getNConfirm());
    //ippNConfirm.setRange(IntParamPanel.VR_POSITIVE_AND_ZERO);
    rbpMapFunction.setText("Map function: ");
    rbpMapFunction.addItem(MapFunction.MF_HALDANE_NAME);
    rbpMapFunction.addItem(MapFunction.MF_KOSAMBI_NAME);
    rbpMapFunction.setSelectedIndex(orderingOptionBean.getMapFunction());
    ippNBootstrap.setText("# iterations of bootstrap test: ");
    ippNBootstrap.setValue(orderingOptionBean.getNBootstrap());
    ippNBootstrap.setRange(IntParamPanel.VR_POSITIVE);
    cbpTargetedGroup.setText("Targeted group(s): ");
    cbpTargetedGroup.addItem("All");
    cbpTargetedGroup.setSelectedIndex(0);
    this.setLayout(gridLayout1);
    gridLayout1.setColumns(1);
    gridLayout1.setRows(6);
    this.add(rbpCriterion, null);
    //this.add(ippMaxIteration, null);
    //this.add(ippNConfirm, null);
    this.add(ippNIteration, null);
    this.add(rbpMapFunction, null);
    this.add(ippNBootstrap, null);
    this.add(cbpTargetedGroup, null);
  }

}
