/*
AntMap: Software for constructing genetic linkage maps using an Ant Colony Optimization (ACO) algorithm
Copyright (C) 2004 National Agricultural and Bio-oriented Research Organization (NARO)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

Hiroyoshi IWATA
Department of Information Science and Technology
National Agricultural Research Center
3-1-1 Kannondai
Tsukuba
Ibaraki 305-8666
JAPAN

ph        +81-298-38-7025
fax       +81-298-38-8551
e-mail    iwatah@affrc.go.jp
*/

package org.agmodel.bioinfo.segregation;

/**
 * @author iwata
 *
 */
public class MarkerTypeGetter {
	/**
	 * Obtain marker type Array
	 * @param ls Arrays of genotypes for multiple loci (Matrix)
	 * @return Marker typei0 - codominant, 1 - dominant (allele a is dominant), 2 - dominant (allele b is dominant))
	 */
	public int [] getMarkerTypeArray(char [][] ls) {
		int [] markerTypeArray = new int [ls.length];
		for(int i = 0; i < ls.length; i++ ) {
			markerTypeArray[i] = getMarkerType(ls[i]);
		}
		return markerTypeArray;
	}

	/**
	 * Obtain marker type
	 * @param l Array of genotypes for the locus
	 * @return Marker typei0 - codominant, 1 - dominant (allele a is dominant), 2 - dominant (allele b is dominant))
	 */
	public int getMarkerType(char [] l) {
		int markerType = MarkerType.MT_CODOM;
		for(int i = 0; i < l.length; i++ ) {
			if(l[i] == 'H') {
				break;
			} else if(l[i] == 'D') {
				markerType = MarkerType.MT_DOM_P1;
				break;
			} else if(l[i] == 'C') {
				markerType = MarkerType.MT_DOM_P2;
				break;
			}
		}
		return markerType;
	}
}
